@@ -29,6 +29,9 @@ class LiveReloadHandler(WebSocketHandler):
     def allow_draft76(self):
         return True
 
+    def check_origin(self, origin):
+        return True
+
     def on_close(self):
         if self in LiveReloadHandler.waiters:
             LiveReloadHandler.waiters.remove(self)
@@ -43,20 +46,33 @@ class LiveReloadHandler(WebSocketHandler):
             logging.error('Error sending message', exc_info=True)
 
     def poll_tasks(self):
-        filepath = self.watcher.examine()
+        filepath, delay = self.watcher.examine()
         if not filepath:
             return
-        logging.info('File %s changed', filepath)
-        self.watch_tasks()
+        reload_time = 3
 
-    def watch_tasks(self):
-        if time.time() - self._last_reload_time < 3:
+        if delay:
+            reload_time = max(3 - delay, 1)
+        if filepath == '__livereload__':
+            reload_time = 0
+
+        if time.time() - self._last_reload_time < reload_time:
             # if you changed lot of files in one time
             # it will refresh too many times
-            logging.info('ignore this reload action')
+            logging.info('Ignore: %s', filepath)
             return
+        if delay:
+            loop = ioloop.IOLoop.current()
+            loop.call_later(delay, self.watch_tasks)
+        else:
+            self.watch_tasks()
 
-        logging.info('Reload %s waiters', len(self.waiters))
+    def watch_tasks(self):
+        logging.info(
+            'Reload %s waiters: %s',
+            len(self.waiters),
+            self.watcher.filepath,
+        )
 
         msg = {
             'command': 'reload',
@@ -106,7 +122,8 @@ class LiveReloadHandler(WebSocketHandler):
 
                 LiveReloadHandler._last_reload_time = time.time()
                 logging.info('Start watching changes')
-                ioloop.PeriodicCallback(self.poll_tasks, 800).start()
+                if not self.watcher.start(self.poll_tasks):
+                    ioloop.PeriodicCallback(self.poll_tasks, 800).start()
 
 
 class LiveReloadJSHandler(RequestHandler):
@@ -128,8 +145,9 @@ class ForceReloadHandler(RequestHandler):
     def get(self):
         msg = {
             'command': 'reload',
-            'path': '*',
-            'liveCSS': True
+            'path': self.get_argument('path', default=None) or '*',
+            'liveCSS': True,
+            'liveImg': True
         }
         for waiter in LiveReloadHandler.waiters:
             try:
@@ -154,7 +172,7 @@ class StaticHandler(RequestHandler):
         elif not os.path.exists(url) and not url.endswith('.html'):
             url += '.html'
 
-        if not os.path.exists(url):
+        if not os.path.isfile(url):
             return None
         return url
 
@@ -178,8 +196,12 @@ class StaticHandler(RequestHandler):
         self.mime_type = mime_type
         self.set_header('Content-Type', mime_type)
 
-        with open(filepath, 'r') as f:
-            data = f.read()
+        if mime_type.startswith('text'):
+            with open(filepath, 'r') as f:
+                data = f.read()
+        else:
+            with open(filepath, 'rb') as f:
+                data = f.read()
 
         hasher = hashlib.sha1()
         hasher.update(to_bytes(data))
