@@ -11,6 +11,10 @@
 import os
 import logging
 from subprocess import Popen, PIPE
+import time
+import threading
+import webbrowser
+
 from tornado import escape
 from tornado.wsgi import WSGIContainer
 from tornado.ioloop import IOLoop
@@ -28,9 +32,11 @@ def shell(command, output=None, mode='w'):
 
     You can add a shell command::
 
-        server.watch('style.less', shell('lessc style.less', output='style.css'))
+        server.watch(
+            'style.less', shell('lessc style.less', output='style.css')
+        )
 
-    :param command: a shell command
+    :param command: a shell command, string or list
     :param output: output stdout to the given file
     :param mode: only works with output, mode ``w`` means write,
                  mode ``a`` means append
@@ -42,7 +48,10 @@ def shell(command, output=None, mode='w'):
         if folder and not os.path.isdir(folder):
             os.makedirs(folder)
 
-    cmd = command.split()
+    if isinstance(command, (list, tuple)):
+        cmd = command
+    else:
+        cmd = command.split()
 
     def run_shell():
         try:
@@ -127,7 +136,9 @@ class Server(object):
 
     :param app: a wsgi application instance
     :param watcher: A Watcher instance, you don't have to initialize
-                    it by yourself
+                    it by yourself. Under Linux, you will want to install
+                    pyinotify and use INotifyWatcher() to avoid wasted
+                    CPU usage.
     """
     def __init__(self, app=None, watcher=None):
         self.app = app
@@ -137,7 +148,7 @@ class Server(object):
             watcher = Watcher()
         self.watcher = watcher
 
-    def watch(self, filepath, func=None):
+    def watch(self, filepath, func=None, delay=None):
         """Add the given filepath for watcher list.
 
         Once you have intialized a server, watch file changes before
@@ -154,11 +165,12 @@ class Server(object):
         :param func: the function to be called, it can be a string of
                      shell command, or any callable object without
                      parameters
+        :param delay: delay a certain seconds to send the reload message
         """
         if isinstance(func, text_types):
             func = shell(func)
 
-        self.watcher.watch(filepath, func)
+        self.watcher.watch(filepath, func, delay)
 
     def application(self, debug=True):
         LiveReloadHandler.watcher = self.watcher
@@ -179,12 +191,14 @@ class Server(object):
             )
         return Application(handlers=handlers, debug=debug)
 
-    def serve(self, port=None, host=None, root=None, debug=True):
+    def serve(self, port=None, host=None, root=None, debug=True,
+              open_url=False, reload_delay=2):
         """Start serve the server with the given port.
 
         :param port: serve on this port, default is 5500
         :param host: serve on this hostname, default is 0.0.0.0
         :param root: serve static on this root directory
+        :param open_url: open system browser
         """
         if root:
             self.root = root
@@ -195,8 +209,19 @@ class Server(object):
 
         self.application(debug=debug).listen(self.port, address=host)
         logging.getLogger().setLevel(logging.INFO)
-        print('Serving on 127.0.0.1:%s' % self.port)
+
+        host = host or '127.0.0.1'
+        print('Serving on http://%s:%s' % (host, self.port))
+
+        # Async open web browser after 5 sec timeout
+        if open_url:
+            def opener():
+                time.sleep(5)
+                webbrowser.open('http://%s:%s' % (host, self.port))
+            threading.Thread(target=opener).start()
+
         try:
+            self.watcher._changes.append(('__livereload__', reload_delay))
             IOLoop.instance().start()
         except KeyboardInterrupt:
             print('Shutting down...')
